package io.malloc.ccc.cc;

import com.wolfssl.wolfcrypt.Aes;
import io.malloc.ccc.nc.Util;


public class AesCipher implements ICipher {

  final int PADDING_SIZE = 16;

  private Aes     aes;
  private byte[]  iv;
  private byte[]  key;
  private boolean isEnc;


  public AesCipher(byte[] key, byte[] iv, boolean isEnc) throws NastySizeException {
    init(key, iv, isEnc);
  }

  @Override
  public void init(byte[] key, byte[] iv, boolean isEnc) throws NastySizeException {
    checkSizes(key.length, iv.length);
    this.iv     = iv;
    this.key    = key;
    this.isEnc  = isEnc;
    if(isEnc) {
      System.out.println("Aes.ENCRYPT_MODE");
      aes = new Aes(key, iv, Aes.ENCRYPT_MODE);
//      aes.setKey(iv, key, Aes.ENCRYPT_MODE);
    }
    else {
      System.out.println("Aes.DECRYPT_MODE");
      aes = new Aes(key, iv, Aes.DECRYPT_MODE);
//      aes.setKey(iv, key, Aes.DECRYPT_MODE);
    }
    System.out.println("iv=="+Util.b2h(iv));
    System.out.println("key=="+Util.b2h(key));
  }

  @Override
  public byte[] enc(byte[] data) throws NotPayingAttentionException {
    byte[] result;
    System.out.println("data=="+Util.b2h(data));
    if(isEnc) {
      byte[] newData = padData(data);
      System.out.println("newData=="+Util.b2h(newData));
      result = aes.update(newData, 0, newData.length);
      System.out.println("result=="+Util.b2h(result));
    }
    else {
      throw new NotPayingAttentionException("you are encrypting while in decrypting mode");
    }
    return result;
  }

  @Override
  public byte[] dec(byte[] data) throws NotPayingAttentionException {
    byte[] result;
    if(data.length % 16 != 0)
      throw new NotPayingAttentionException("data is not multiple of 16 bytes");
    if(!isEnc) {
      result = aes.update(data, 0, data.length);
      System.out.println("dec() result=="+Util.b2h(result)+"; length=="+result.length);
      result = unPadData(result);
      System.out.println("dec() result=="+Util.b2h(result)+"; length=="+result.length);
    }
    else {
      throw new NotPayingAttentionException("you are decrypting while in encrypting mode");
    }
    return result;
  }

  public void destroy() {
    aes.releaseNativeStruct();
  }

  private void checkSizes(int keyLen, int ivLen) throws NastySizeException {
    if (keyLen != 16 && keyLen != 24 && keyLen != 32)
      throw new NastySizeException("Key isn't 16, 24, 32 bytes; length is "+keyLen+" bytes");
    if (ivLen != 16 && ivLen != 24 && ivLen != 32)
      throw new NastySizeException("IV isn't 16 bytes; IV is "+ivLen+" bytes");
  }

  private byte[] padData(byte[] data) {
    int remainder = data.length % PADDING_SIZE;
    System.out.println("[J3G] data.length % 16 == "+remainder);
    if(remainder != 0) {
      remainder = PADDING_SIZE - remainder;
      byte[] newData = new byte[data.length+remainder+PADDING_SIZE];
      System.arraycopy(data, 0, newData, 0, data.length);
      for(int i=data.length; i<newData.length; i++)
        newData[i] = (byte)remainder;

      return newData;
    }
    return data;
  }

  private byte[] unPadData(byte[] data) {
    byte b = data[data.length-1];
    System.out.println("padding=="+b);
    byte[] newData = new byte[data.length-(b+PADDING_SIZE)];
    System.arraycopy(data, 0, newData, 0, newData.length);
    return newData;
  }

}
